<?php
/*******************************************************
 * Only these origins will be allowed to upload images *
 ******************************************************/

require_once(__DIR__ . '/../../vendor/autoload.php');

use IMATHUZH\Qfq\Core\QuickFormQuery;
use IMATHUZH\Qfq\Core\Store\Store;

reset($_FILES);
$temp = current($_FILES);
$qfq = array();

// Get QFQ variables and the needed System Store
try {
    try {
        $qfq = new QuickFormQuery(['bodytext' => '']);
    } catch (\UserReportException $e) {
        $answer[API_MESSAGE] = $e->formatMessage();
    } catch (\CodeException $e) {
        $answer[API_MESSAGE] = $e->formatMessage();
    } catch (\DbException $e) {
        $answer[API_MESSAGE] = $e->formatMessage();
    }
} catch (\Exception $e) {
    $answer[API_MESSAGE] = "Generic Exception: " . $e->getMessage();
}

$httpOrigin = Store::getVar(SYSTEM_HTTP_ORIGIN, STORE_SYSTEM, SANITIZE_ALLOW_ALL);

if (is_uploaded_file($temp['tmp_name'])) {
    if (isset($httpOrigin)) {
        // same-origin requests won't set an origin. If the origin is set, it must be valid.
        if ($httpOrigin === $_SERVER['HTTP_ORIGIN']) {
            header('Access-Control-Allow-Origin: ' . $httpOrigin);
        } else {
            header(HTTP_403_ORIGIN_DENIED);
            return;
        }
    }

    /*
      If your script needs to receive cookies, set images_upload_credentials : true in
      the configuration and enable the following two headers.
    */
    // header('Access-Control-Allow-Credentials: true');
    // header('P3P: CP="There is no P3P policy."');

    // Sanitize input
    if (preg_match("/([^\w\s\d\-_~,;:\[\]\(\).])|([\.]{2,})/", $temp['name'])) {
        header(HTTP_400_INVALID_FILE_NAME);
        return;
    }

    // Verify extension
    // Split between 'Media Type' and 'Media Subtype'
    $fileMimeTypeSplitted = explode('/', $temp['type'], 2);
    if ($fileMimeTypeSplitted[0] !== 'image') {
        header(HTTP_400_INVALID_EXTENSION);
        return;
    }

    // Base folderpath should be with user input changeable -> take parameters from form
    $folderPath = '';
    $folderPath = Store::getVar(SYSTEM_IMAGE_UPLOAD_DIR, STORE_SIP, SANITIZE_ALLOW_ALL);
    $folderPath = base64_decode($folderPath);


    // Accept upload if there was no origin, or if it is an accepted origin
    $imageUploadDir = Store::getVar(SYSTEM_IMAGE_UPLOAD_DIR, STORE_SYSTEM, SANITIZE_ALLOW_ALL);

    //set user given path if exists
    if ($folderPath != '') {
        $imageUploadDir = $folderPath;
    }

    $imageFolder = "../../../../../" . $imageUploadDir;

    @mkdir($imageFolder);

    // Name should be unique, add to end of name number if exists to not overwrite it
    $changedFileName = getUniqueFileName($imageFolder, $temp['name']);

    $filetowrite = $imageFolder . '/' . $changedFileName;
    $filePath = $imageUploadDir . '/' . $changedFileName;
    move_uploaded_file($temp['tmp_name'], $filetowrite);

    // Respond to the successful upload with JSON.
    // Use a location key to specify the path to the saved image resource.
    // { location : '/your/uploaded/image/file'}
    echo json_encode(array('location' => $filePath));
} else {
    // Notify editor that the upload failed
    header(HTTP_500_SERVER_ERROR);
}

/**
 * @param $destinationFolderPath
 * @param $fileName
 * @return array|string|string[]
 */
    function getUniqueFileName ($destinationFolderPath, $fileName){
        $allFiles = scandir($destinationFolderPath);
        $changedFileName = pathinfo($fileName, PATHINFO_FILENAME);
        $extension = pathinfo($fileName, PATHINFO_EXTENSION);
        $originalFileName = pathinfo($fileName, PATHINFO_FILENAME);
        $count = 0;

        for ($i = 0; $i < count($allFiles); $i++){
            if (pathinfo($allFiles[$i],PATHINFO_FILENAME) === $changedFileName) {
                $count++;
                $changedFileName = $originalFileName.'-'.$count;
                $i = 0;
            }
        }

        if ($extension != '') {
            $changedFileName .= '.'.$extension;
        }
        return $changedFileName;
    }
?>